﻿using UnityEngine;
using System.Collections;
using UnityEngine.Networking;

public class CharacterControlBase : NetworkBehaviour
{
    [SyncVar(hook = "UpdateHealthBar")]
	public int Health = 60;
	public int Damage1 = 20;
	public int Damage2 = 30;
	public int Damage3 = 40;
	public float DamageRange = 1.0f;
    public GameObject healthBarImage;

	protected Animator animator;
    protected NetworkAnimator netAnimator;
    [SyncVar]
	public bool isDead = false;
	protected float ResurrectionTime = 3.0f;
	protected int MaxHealth = 60;

	void Start () {
		animator = GetComponent<Animator> ();
        netAnimator = GetComponent<NetworkAnimator>();
		//Reborn ();
        isDead = false;
        RpcReborn(transform.position, transform.rotation);
		StartImpl ();
	}

	protected virtual void StartImpl(){

	}

    void UpdateHealthBar(int health) {
        if (healthBarImage == null) {
            Debug.LogWarning("No health bar image connected!");
            return;
        }
        healthBarImage.transform.localScale = new Vector3((float)Mathf.Min(Mathf.Max(health, 0), MaxHealth) / (float)MaxHealth, 1, 1);
    }

	void Update ()
	{
		if (isDead) {
			return;
		}
		UpdateImpl();
	}

	protected virtual void UpdateImpl(){
		
	}

    protected virtual void DieImpl()
    {

    }

    [ClientRpc]
	protected void RpcDie()
	{
		animator.enabled = false;
		Rigidbody ownBody = GetComponent<Rigidbody>();
		Rigidbody[] ragdollBodies;
		ragdollBodies = GetComponentsInChildren<Rigidbody> ();
		foreach (Rigidbody body in ragdollBodies) {
			if(body != ownBody)
				body.isKinematic = false;
		}
        DieImpl();
	}

    [ClientRpc]
    protected void RpcReborn(Vector3 targetPos, Quaternion targetRot)
    {
        Health = MaxHealth;
		animator.enabled = true;
		Rigidbody ownBody = GetComponent<Rigidbody>();
		Rigidbody[] ragdollBodies;
		ragdollBodies = GetComponentsInChildren<Rigidbody> ();
		foreach (Rigidbody body in ragdollBodies) {
			if(body != ownBody)
				body.isKinematic = true;
		}
    }

	protected void Reborn()
	{
		isDead = false;
        Transform targetTransform = NetworkManager.singleton.GetStartPosition();
        RpcReborn(targetTransform.position, targetTransform.rotation);
	}

    [Command]
	public void CmdTakeDamage(int damage)
	{
		if (isDead) return;
		Health -= damage;
        if (Health <= 0) {
            isDead = true;
            RpcDie();
            if (ResurrectionTime >= 0) {  // < 0: no resurrection
                Invoke("Reborn", ResurrectionTime);
            }
        }
	}

	protected void hit(int damage)
	{
        if (!isServer) return;
		Debug.Log ("Hit\n");
		GameObject[] paladins = GameObject.FindGameObjectsWithTag ("Player");
		foreach (GameObject p in paladins)
		{
			if (p == gameObject)
				continue;
			//Debug.Log ("Checking paladin\n");
			Vector3 dir = p.transform.position - transform.position;
			//Debug.Log ("Paladin range" + dir.magnitude);
			if (dir.magnitude < DamageRange)
			{
				//Debug.Log ("Paladin in range\n");
				if (Vector3.Angle (dir.normalized, transform.forward) < 45)
				{
					//Debug.Log ("paladin hit\n");
					p.GetComponent<CharacterControlBase> ().CmdTakeDamage (damage);
				}
			}
		}
		GameObject[] trolls = GameObject.FindGameObjectsWithTag ("Troll");
		foreach (GameObject t in trolls)
		{
			if (t == gameObject)
				continue;
			//Debug.Log ("Checking troll\n");
			Vector3 dir = t.transform.position - transform.position;
			//Debug.Log ("Troll range" + dir.magnitude);
			if (dir.magnitude < DamageRange)
			{
				//Debug.Log ("Troll in range\n");
				if (Vector3.Angle (dir.normalized, transform.forward) < 45)
				{
					//Debug.Log ("Troll hit\n");
					t.GetComponent<CharacterControlBase> ().CmdTakeDamage (damage);
				}
			}
		}
	}

	public void AnimationHitpoint(int mode)
	{
        if (!isServer) return;
		switch (mode) {
		case 0:
			hit (Damage1);
			break;
		case 1:
			hit (Damage2);
			break;
		case 2:
			hit (Damage3);
			break;
		}
	}
}

public class CharacterControl : CharacterControlBase {	

	public CharacterControl()
	{
		MaxHealth = Health = 60;
		Damage1 = 20;
		Damage2 = 20;
		Damage3 = 40;
		ResurrectionTime = 10.0f;
		DamageRange = 2.0f;
	}

	protected override void StartImpl () {
		Cursor.lockState = CursorLockMode.Confined;
	}

	protected override void UpdateImpl() {
        if (!isLocalPlayer) return; // csak a kliens saját játékos objektumán fusson le az Update

		float runAxis = Input.GetAxis ("Run");
		float forwardAxis = Input.GetAxis ("Vertical");
		animator.SetFloat ("forward", forwardAxis * (1.0f + runAxis));
		float sideAxis = Input.GetAxis ("Horizontal");
		animator.SetFloat ("side", sideAxis * (1.0f + runAxis));

		float mouseX = Input.GetAxis ("Mouse X");
		transform.Rotate(new Vector3(0,mouseX*Time.deltaTime*1000,0));

		if (Input.GetKeyDown (KeyCode.Space))
			netAnimator.SetTrigger ("JumpTriggered");
		else if (Input.GetKeyDown ("q") || Input.GetMouseButtonDown(0)) {
			animator.SetInteger ("AttackType", 0);
			netAnimator.SetTrigger ("AttackTriggered");
		} else if (Input.GetKeyDown ("e") || Input.GetMouseButtonDown(1)) {
			animator.SetInteger ("AttackType", 1);
			netAnimator.SetTrigger ("AttackTriggered");
		}
        else if (Input.GetKeyDown("r") || Input.GetMouseButtonDown(2))
        {
            animator.SetInteger("AttackType", 2);
            netAnimator.SetTrigger("AttackTriggered");
        }
        else if (Input.GetKeyDown ("k")) {
            CmdTakeDamage(MaxHealth);
		}
	}
}
